/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class PredicatesTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testAlwaysFalse_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAlwaysFalse_apply();
}

public void testAlwaysFalse_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAlwaysFalse_equality();
}

public void testAlwaysTrue_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAlwaysTrue_apply();
}

public void testAlwaysTrue_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAlwaysTrue_equality();
}

public void testAnd_applyBinary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_applyBinary();
}

public void testAnd_applyIterable() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_applyIterable();
}

public void testAnd_applyNoArgs() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_applyNoArgs();
}

public void testAnd_applyOneArg() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_applyOneArg();
}

public void testAnd_applyTernary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_applyTernary();
}

public void testAnd_arrayDefensivelyCopied() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_arrayDefensivelyCopied();
}

public void testAnd_equalityBinary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_equalityBinary();
}

public void testAnd_equalityIterable() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_equalityIterable();
}

public void testAnd_equalityNoArgs() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_equalityNoArgs();
}

public void testAnd_equalityOneArg() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_equalityOneArg();
}

public void testAnd_equalityTernary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_equalityTernary();
}

public void testAnd_iterableDefensivelyCopied() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_iterableDefensivelyCopied();
}

public void testAnd_listDefensivelyCopied() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testAnd_listDefensivelyCopied();
}

public void testCompose() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testCompose();
}

public void testHashCodeForBooleanOperations() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testHashCodeForBooleanOperations();
}

public void testIn_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIn_apply();
}

public void testIn_compilesWithExplicitSupertype() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIn_compilesWithExplicitSupertype();
}

public void testIn_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIn_equality();
}

public void testIn_handlesClassCastException() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIn_handlesClassCastException();
}

public void testIn_handlesNullPointerException() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIn_handlesNullPointerException();
}

public void testIsEqualToNull_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIsEqualToNull_apply();
}

public void testIsEqualToNull_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIsEqualToNull_equality();
}

public void testIsEqualTo_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIsEqualTo_apply();
}

public void testIsEqualTo_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIsEqualTo_equality();
}

public void testIsNull_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIsNull_apply();
}

public void testIsNull_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testIsNull_equality();
}

public void testNotNull_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testNotNull_apply();
}

public void testNotNull_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testNotNull_equality();
}

public void testNot_apply() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testNot_apply();
}

public void testNot_equality() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testNot_equality();
}

public void testNot_equalityForNotOfKnownValues() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testNot_equalityForNotOfKnownValues();
}

public void testOr_applyBinary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_applyBinary();
}

public void testOr_applyIterable() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_applyIterable();
}

public void testOr_applyNoArgs() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_applyNoArgs();
}

public void testOr_applyOneArg() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_applyOneArg();
}

public void testOr_applyTernary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_applyTernary();
}

public void testOr_arrayDefensivelyCopied() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_arrayDefensivelyCopied();
}

public void testOr_equalityBinary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_equalityBinary();
}

public void testOr_equalityIterable() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_equalityIterable();
}

public void testOr_equalityNoArgs() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_equalityNoArgs();
}

public void testOr_equalityOneArg() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_equalityOneArg();
}

public void testOr_equalityTernary() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_equalityTernary();
}

public void testOr_iterableDefensivelyCopied() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_iterableDefensivelyCopied();
}

public void testOr_listDefensivelyCopied() throws Exception {
  com.google.common.base.PredicatesTest testCase = new com.google.common.base.PredicatesTest();
  testCase.testOr_listDefensivelyCopied();
}
}
