/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class MultimapsTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testAsMap_listMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testAsMap_listMultimap();
}

public void testAsMap_multimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testAsMap_multimap();
}

public void testAsMap_setMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testAsMap_setMultimap();
}

public void testAsMap_sortedSetMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testAsMap_sortedSetMultimap();
}

public void testFilteredKeysListMultimapGetBadValue() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testFilteredKeysListMultimapGetBadValue();
}

public void testFilteredKeysSetMultimapGetBadValue() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testFilteredKeysSetMultimapGetBadValue();
}

public void testFilteredKeysSetMultimapReplaceValues() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testFilteredKeysSetMultimapReplaceValues();
}

public void testFlatteningToMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testFlatteningToMultimap();
}

public void testForMap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testForMap();
}

public void testForMapAsMap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testForMapAsMap();
}

public void testForMapGetIteration() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testForMapGetIteration();
}

public void testForMapRemoveAll() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testForMapRemoveAll();
}

public void testIndex() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testIndex();
}

public void testIndexIterator() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testIndexIterator();
}

public void testIndex_nullKey() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testIndex_nullKey();
}

public void testIndex_nullValue() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testIndex_nullValue();
}

public void testIndex_ordering() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testIndex_ordering();
}

public void testInvertFrom() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testInvertFrom();
}

public void testMultimapCollectorGenerics() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testMultimapCollectorGenerics();
}

public void testNewListMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewListMultimap();
}

public void testNewMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewMultimap();
}

public void testNewMultimapValueCollectionMatchesList() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewMultimapValueCollectionMatchesList();
}

public void testNewMultimapValueCollectionMatchesNavigableSet() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewMultimapValueCollectionMatchesNavigableSet();
}

public void testNewMultimapWithCollectionRejectingNegativeElements() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewMultimapWithCollectionRejectingNegativeElements();
}

public void testNewSetMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewSetMultimap();
}

public void testNewSortedSetMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testNewSortedSetMultimap();
}

public void testSynchronizedMultimapSampleCodeCompilation() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testSynchronizedMultimapSampleCodeCompilation();
}

public void testToMultimap() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testToMultimap();
}

public void testUnmodifiableArrayListMultimapRandomAccess() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableArrayListMultimapRandomAccess();
}

public void testUnmodifiableLinkedListMultimapRandomAccess() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableLinkedListMultimapRandomAccess();
}

public void testUnmodifiableListMultimapShortCircuit() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableListMultimapShortCircuit();
}

public void testUnmodifiableMultimapEntries() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableMultimapEntries();
}

public void testUnmodifiableMultimapIsView() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableMultimapIsView();
}

public void testUnmodifiableMultimapShortCircuit() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableMultimapShortCircuit();
}

public void testUnmodifiableSetMultimapShortCircuit() throws Exception {
  com.google.common.collect.MultimapsTest testCase = new com.google.common.collect.MultimapsTest();
  testCase.testUnmodifiableSetMultimapShortCircuit();
}
}
