/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class EmptyImmutableTableTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testCellSet() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testCellSet();
}

public void testClear() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testClear();
}

public void testColumn() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testColumn();
}

public void testColumnKeySet() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testColumnKeySet();
}

public void testColumnMap() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testColumnMap();
}

public void testConsistentHashCode() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testConsistentHashCode();
}

public void testConsistentToString() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testConsistentToString();
}

public void testContains() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testContains();
}

public void testContainsColumn() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testContainsColumn();
}

public void testContainsRow() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testContainsRow();
}

public void testContainsValue() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testContainsValue();
}

public void testEqualsObject() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testEqualsObject();
}

public void testGet() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testGet();
}

public void testHashCode() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testHashCode();
}

public void testIsEmpty() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testIsEmpty();
}

public void testPut() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testPut();
}

public void testPutAll() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testPutAll();
}

public void testRemove() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testRemove();
}

public void testRow() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testRow();
}

public void testRowKeySet() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testRowKeySet();
}

public void testRowMap() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testRowMap();
}

public void testSize() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testSize();
}

public void testToString() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testToString();
}

public void testValues() throws Exception {
  com.google.common.collect.EmptyImmutableTableTest testCase = new com.google.common.collect.EmptyImmutableTableTest();
  testCase.testValues();
}
}
