/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.util.concurrent;
public class AtomicLongMapTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.util.concurrent.testModule";
}
public void testAddAndGet() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testAddAndGet();
}

public void testAddAndGet_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testAddAndGet_zero();
}

public void testClear() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testClear();
}

public void testCreate_map() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testCreate_map();
}

public void testDecrementAndGet() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testDecrementAndGet();
}

public void testDecrementAndGet_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testDecrementAndGet_zero();
}

public void testEmpty() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testEmpty();
}

public void testGetAndAdd() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testGetAndAdd();
}

public void testGetAndAdd_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testGetAndAdd_zero();
}

public void testGetAndDecrement() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testGetAndDecrement();
}

public void testGetAndDecrement_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testGetAndDecrement_zero();
}

public void testGetAndIncrement() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testGetAndIncrement();
}

public void testGetAndIncrement_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testGetAndIncrement_zero();
}

public void testIncrementAndGet() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testIncrementAndGet();
}

public void testIncrementAndGet_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testIncrementAndGet_zero();
}

public void testPut() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testPut();
}

public void testPutAll() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testPutAll();
}

public void testPutIfAbsent() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testPutIfAbsent();
}

public void testPutIfAbsent_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testPutIfAbsent_zero();
}

public void testPut_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testPut_zero();
}

public void testRemove() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testRemove();
}

public void testRemoveIfZero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testRemoveIfZero();
}

public void testRemoveValue() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testRemoveValue();
}

public void testRemoveValue_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testRemoveValue_zero();
}

public void testRemoveZeros() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testRemoveZeros();
}

public void testRemove_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testRemove_zero();
}

public void testReplace() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testReplace();
}

public void testReplace_zero() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testReplace_zero();
}

public void testSerialization() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testSerialization();
}

public void testSum() throws Exception {
  com.google.common.util.concurrent.AtomicLongMapTest testCase = new com.google.common.util.concurrent.AtomicLongMapTest();
  testCase.testSum();
}
}
