/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class ConverterTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testAndThen() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testAndThen();
}

public void testApply() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testApply();
}

public void testConvertAllIsView() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testConvertAllIsView();
}

public void testConverter() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testConverter();
}

public void testFrom() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testFrom();
}

public void testIdentityConverter() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testIdentityConverter();
}

public void testNullIsNotPassedThrough() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testNullIsNotPassedThrough();
}

public void testNullIsPassedThrough() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testNullIsPassedThrough();
}

public void testReverse() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testReverse();
}

public void testReverseReverse() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testReverseReverse();
}

public void testSerialization_andThen() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testSerialization_andThen();
}

public void testSerialization_from() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testSerialization_from();
}

public void testSerialization_identity() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testSerialization_identity();
}

public void testSerialization_reverse() throws Exception {
  com.google.common.base.ConverterTest testCase = new com.google.common.base.ConverterTest();
  testCase.testSerialization_reverse();
}
}
