/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class EquivalenceTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testEquals() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testEquals();
}

public void testEqualsEquivalent() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testEqualsEquivalent();
}

public void testEquivalentTo() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testEquivalentTo();
}

public void testIdentityEquivalent() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testIdentityEquivalent();
}

public void testOnResultOf() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testOnResultOf();
}

public void testOnResultOf_equals() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testOnResultOf_equals();
}

public void testPairwiseEquivalent() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testPairwiseEquivalent();
}

public void testPairwiseEquivalent_equals() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testPairwiseEquivalent_equals();
}

public void testWrap() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testWrap();
}

public void testWrap_get() throws Exception {
  com.google.common.base.EquivalenceTest testCase = new com.google.common.base.EquivalenceTest();
  testCase.testWrap_get();
}
}
