/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.net;
public class MediaTypeTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.net.testModule";
}
public void testCreateApplicationType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreateApplicationType();
}

public void testCreateAudioType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreateAudioType();
}

public void testCreateImageType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreateImageType();
}

public void testCreateTextType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreateTextType();
}

public void testCreateVideoType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreateVideoType();
}

public void testCreate_invalidSubtype() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreate_invalidSubtype();
}

public void testCreate_invalidType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreate_invalidType();
}

public void testCreate_wildcardTypeDeclaredSubtype() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testCreate_wildcardTypeDeclaredSubtype();
}

public void testEquals() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testEquals();
}

public void testGetCharset() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetCharset();
}

public void testGetCharset_illegalCharset() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetCharset_illegalCharset();
}

public void testGetCharset_tooMany() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetCharset_tooMany();
}

public void testGetCharset_unsupportedCharset() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetCharset_unsupportedCharset();
}

public void testGetParameters() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetParameters();
}

public void testGetSubtype() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetSubtype();
}

public void testGetType() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testGetType();
}

public void testHasWildcard() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testHasWildcard();
}

public void testIs() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testIs();
}

public void testParse_badInput() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testParse_badInput();
}

public void testParse_empty() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testParse_empty();
}

public void testToString() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testToString();
}

public void testWithCharset() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithCharset();
}

public void testWithParameter() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParameter();
}

public void testWithParameter_invalidAttribute() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParameter_invalidAttribute();
}

public void testWithParameters() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParameters();
}

public void testWithParametersIterable() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParametersIterable();
}

public void testWithParametersIterable_invalidAttribute() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParametersIterable_invalidAttribute();
}

public void testWithParametersIterable_nullValue() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParametersIterable_nullValue();
}

public void testWithParameters_invalidAttribute() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithParameters_invalidAttribute();
}

public void testWithoutParameters() throws Exception {
  com.google.common.net.MediaTypeTest testCase = new com.google.common.net.MediaTypeTest();
  testCase.testWithoutParameters();
}
}
