/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.primitives;
public class UnsignedLongTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.primitives.testModule";
}
public void testAsUnsignedAndLongValueAreInverses() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testAsUnsignedAndLongValueAreInverses();
}

public void testAsUnsignedBigIntegerValue() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testAsUnsignedBigIntegerValue();
}

public void testCompare() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testCompare();
}

public void testDivideByZeroThrows() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testDivideByZeroThrows();
}

public void testDividedBy() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testDividedBy();
}

public void testDoubleValue() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testDoubleValue();
}

public void testFloatValue() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testFloatValue();
}

public void testIntValue() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testIntValue();
}

public void testMinus() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testMinus();
}

public void testMod() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testMod();
}

public void testModByZero() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testModByZero();
}

public void testPlus() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testPlus();
}

public void testTimes() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testTimes();
}

public void testToString() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testToString();
}

public void testToStringRadixQuick() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testToStringRadixQuick();
}

public void testValueOfBigInteger() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testValueOfBigInteger();
}

public void testValueOfLong() throws Exception {
  com.google.common.primitives.UnsignedLongTest testCase = new com.google.common.primitives.UnsignedLongTest();
  testCase.testValueOfLong();
}
}
