/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class CharMatcherTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testAllMatches() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testAllMatches();
}

public void testAnyAndNone_logicalOps() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testAnyAndNone_logicalOps();
}

public void testCollapse() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testCollapse();
}

public void testCollapse_any() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testCollapse_any();
}

public void testEmpty() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testEmpty();
}

public void testGeneral() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testGeneral();
}

public void testNoMatches() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testNoMatches();
}

public void testPrecomputedOptimizations() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testPrecomputedOptimizations();
}

public void testReplaceFrom() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testReplaceFrom();
}

public void testToString() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testToString();
}

public void testTrimAndCollapse() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testTrimAndCollapse();
}

public void testTrimFrom() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testTrimFrom();
}

public void testTrimLeadingFrom() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testTrimLeadingFrom();
}

public void testTrimTrailingFrom() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testTrimTrailingFrom();
}

public void testWhitespaceBreakingWhitespaceSubset() throws Exception {
  com.google.common.base.CharMatcherTest testCase = new com.google.common.base.CharMatcherTest();
  testCase.testWhitespaceBreakingWhitespaceSubset();
}
}
