/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class StopwatchTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testCreateStarted() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testCreateStarted();
}

public void testCreateUnstarted() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testCreateUnstarted();
}

public void testElapsed_micros() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testElapsed_micros();
}

public void testElapsed_millis() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testElapsed_millis();
}

public void testElapsed_multipleSegments() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testElapsed_multipleSegments();
}

public void testElapsed_notRunning() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testElapsed_notRunning();
}

public void testElapsed_whileRunning() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testElapsed_whileRunning();
}

public void testInitialState() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testInitialState();
}

public void testReset_new() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testReset_new();
}

public void testReset_whileRunning() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testReset_whileRunning();
}

public void testStart() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testStart();
}

public void testStart_whileRunning() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testStart_whileRunning();
}

public void testStop() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testStop();
}

public void testStop_alreadyStopped() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testStop_alreadyStopped();
}

public void testStop_new() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testStop_new();
}

public void testToString() throws Exception {
  com.google.common.base.StopwatchTest testCase = new com.google.common.base.StopwatchTest();
  testCase.testToString();
}
}
