/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.primitives;
public class CharsTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.primitives.testModule";
}
public void testAsListEmpty() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testAsListEmpty();
}

public void testAsList_isAView() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testAsList_isAView();
}

public void testAsList_subList_toArray_roundTrip() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testAsList_subList_toArray_roundTrip();
}

public void testAsList_toArray_roundTrip() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testAsList_toArray_roundTrip();
}

public void testCheckedCast() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testCheckedCast();
}

public void testCompare() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testCompare();
}

public void testConcat() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testConcat();
}

public void testConstrainToRange() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testConstrainToRange();
}

public void testContains() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testContains();
}

public void testEnsureCapacity() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testEnsureCapacity();
}

public void testEnsureCapacity_fail() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testEnsureCapacity_fail();
}

public void testHashCode() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testHashCode();
}

public void testIndexOf() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testIndexOf();
}

public void testIndexOf_arrayTarget() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testIndexOf_arrayTarget();
}

public void testJoin() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testJoin();
}

public void testLastIndexOf() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testLastIndexOf();
}

public void testLexicographicalComparator() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testLexicographicalComparator();
}

public void testMax() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testMax();
}

public void testMax_noArgs() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testMax_noArgs();
}

public void testMin() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testMin();
}

public void testMin_noArgs() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testMin_noArgs();
}

public void testReverse() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testReverse();
}

public void testReverseIndexed() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testReverseIndexed();
}

public void testSaturatedCast() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testSaturatedCast();
}

public void testSortDescending() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testSortDescending();
}

public void testSortDescendingIndexed() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testSortDescendingIndexed();
}

public void testToArray() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testToArray();
}

public void testToArray_threadSafe() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testToArray_threadSafe();
}

public void testToArray_withNull() throws Exception {
  com.google.common.primitives.CharsTest testCase = new com.google.common.primitives.CharsTest();
  testCase.testToArray_withNull();
}
}
