/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class TreeMultimapNaturalTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testComparators() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testComparators();
}

public void testCreateFromHashMultimap() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testCreateFromHashMultimap();
}

public void testCreateFromSortedSetMultimap() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testCreateFromSortedSetMultimap();
}

public void testCreateFromTreeMultimap() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testCreateFromTreeMultimap();
}

public void testMultimapConstructor() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testMultimapConstructor();
}

public void testOrderedAsMapEntries() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testOrderedAsMapEntries();
}

public void testOrderedEntries() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testOrderedEntries();
}

public void testOrderedGet() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testOrderedGet();
}

public void testOrderedKeySet() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testOrderedKeySet();
}

public void testOrderedValues() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testOrderedValues();
}

public void testTailSetClear() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testTailSetClear();
}

public void testToString() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testToString();
}

public void testTreeMultimapAsMapSorted() throws Exception {
  com.google.common.collect.TreeMultimapNaturalTest testCase = new com.google.common.collect.TreeMultimapNaturalTest();
  testCase.testTreeMultimapAsMapSorted();
}
}
