/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class AsciiTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testCharsIgnored() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testCharsIgnored();
}

public void testCharsLower() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testCharsLower();
}

public void testCharsUpper() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testCharsUpper();
}

public void testEqualsIgnoreCase() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testEqualsIgnoreCase();
}

public void testToLowerCase() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testToLowerCase();
}

public void testToUpperCase() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testToUpperCase();
}

public void testTruncate() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testTruncate();
}

public void testTruncateIllegalArguments() throws Exception {
  com.google.common.base.AsciiTest testCase = new com.google.common.base.AsciiTest();
  testCase.testTruncateIllegalArguments();
}
}
