/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class LinkedHashMultimapTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testCreate() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testCreate();
}

public void testCreateFromIllegalSizes() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testCreateFromIllegalSizes();
}

public void testCreateFromMultimap() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testCreateFromMultimap();
}

public void testCreateFromSizes() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testCreateFromSizes();
}

public void testEntriesSpliterator() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testEntriesSpliterator();
}

public void testKeysSpliterator() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testKeysSpliterator();
}

public void testKeysToString_ordering() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testKeysToString_ordering();
}

public void testOrderingReadOnly() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testOrderingReadOnly();
}

public void testOrderingSynchronized() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testOrderingSynchronized();
}

public void testOrderingUnmodifiable() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testOrderingUnmodifiable();
}

public void testOrderingUpdates() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testOrderingUpdates();
}

public void testPutMultimapOrdered() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testPutMultimapOrdered();
}

public void testToString() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testToString();
}

public void testToStringNullExact() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testToStringNullExact();
}

public void testValueSetHashTableExpansion() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testValueSetHashTableExpansion();
}

public void testValuesSpliterator() throws Exception {
  com.google.common.collect.LinkedHashMultimapTest testCase = new com.google.common.collect.LinkedHashMultimapTest();
  testCase.testValuesSpliterator();
}
}
