/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.cache;
public class CacheBuilderGwtTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.cache.testModule";
}
public void testAsMapEntrySet() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMapEntrySet();
}

public void testAsMapKeySet() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMapKeySet();
}

public void testAsMapKeySet_contains() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMapKeySet_contains();
}

public void testAsMapValues_contains() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMapValues_contains();
}

public void testAsMapValues_iteratorRemove() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMapValues_iteratorRemove();
}

public void testAsMap_containsKey() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMap_containsKey();
}

public void testAsMap_containsValue() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testAsMap_containsValue();
}

public void testExpireAfterAccess() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testExpireAfterAccess();
}

public void testExpireAfterWrite() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testExpireAfterWrite();
}

public void testExpireAfterWriteAndAccess() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testExpireAfterWriteAndAccess();
}

public void testInvalidate() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testInvalidate();
}

public void testInvalidateAll() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testInvalidateAll();
}

public void testLoader() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testLoader();
}

public void testLoadingCache() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testLoadingCache();
}

public void testMapMethods() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testMapMethods();
}

public void testPutAll() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testPutAll();
}

public void testRemovalListener() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testRemovalListener();
}

public void testSizeConstraint() throws Exception {
  com.google.common.cache.CacheBuilderGwtTest testCase = new com.google.common.cache.CacheBuilderGwtTest();
  testCase.setUp();
  testCase.testSizeConstraint();
}
}
