/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class CaseFormatTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testConverterToBackward() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testConverterToBackward();
}

public void testConverterToForward() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testConverterToForward();
}

public void testConverter_nullConversions() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testConverter_nullConversions();
}

public void testConverter_serialization() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testConverter_serialization();
}

public void testConverter_toString() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testConverter_toString();
}

public void testIdentity() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testIdentity();
}

public void testLowerCamelToLowerCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerCamelToLowerCamel();
}

public void testLowerCamelToLowerHyphen() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerCamelToLowerHyphen();
}

public void testLowerCamelToLowerUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerCamelToLowerUnderscore();
}

public void testLowerCamelToUpperCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerCamelToUpperCamel();
}

public void testLowerCamelToUpperUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerCamelToUpperUnderscore();
}

public void testLowerHyphenToLowerCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerHyphenToLowerCamel();
}

public void testLowerHyphenToLowerHyphen() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerHyphenToLowerHyphen();
}

public void testLowerHyphenToLowerUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerHyphenToLowerUnderscore();
}

public void testLowerHyphenToUpperCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerHyphenToUpperCamel();
}

public void testLowerHyphenToUpperUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerHyphenToUpperUnderscore();
}

public void testLowerUnderscoreToLowerCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerUnderscoreToLowerCamel();
}

public void testLowerUnderscoreToLowerHyphen() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerUnderscoreToLowerHyphen();
}

public void testLowerUnderscoreToLowerUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerUnderscoreToLowerUnderscore();
}

public void testLowerUnderscoreToUpperCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerUnderscoreToUpperCamel();
}

public void testLowerUnderscoreToUpperUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testLowerUnderscoreToUpperUnderscore();
}

public void testUpperCamelToLowerCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperCamelToLowerCamel();
}

public void testUpperCamelToLowerHyphen() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperCamelToLowerHyphen();
}

public void testUpperCamelToLowerUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperCamelToLowerUnderscore();
}

public void testUpperCamelToUpperCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperCamelToUpperCamel();
}

public void testUpperCamelToUpperUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperCamelToUpperUnderscore();
}

public void testUpperUnderscoreToLowerCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperUnderscoreToLowerCamel();
}

public void testUpperUnderscoreToLowerHyphen() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperUnderscoreToLowerHyphen();
}

public void testUpperUnderscoreToLowerUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperUnderscoreToLowerUnderscore();
}

public void testUpperUnderscoreToUpperCamel() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperUnderscoreToUpperCamel();
}

public void testUpperUnderscoreToUpperUnderscore() throws Exception {
  com.google.common.base.CaseFormatTest testCase = new com.google.common.base.CaseFormatTest();
  testCase.testUpperUnderscoreToUpperUnderscore();
}
}
