/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.net;
public class InternetDomainNameTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.net.testModule";
}
public void testChild() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testChild();
}

public void testEquality() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testEquality();
}

public void testInvalid() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testInvalid();
}

public void testInvalidTopPrivateDomain() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testInvalidTopPrivateDomain();
}

public void testIsValid() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testIsValid();
}

public void testParent() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testParent();
}

public void testParentChild() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testParentChild();
}

public void testPublicSuffix() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testPublicSuffix();
}

public void testPublicSuffixExclusion() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testPublicSuffixExclusion();
}

public void testPublicSuffixMultipleUnders() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testPublicSuffixMultipleUnders();
}

public void testRegistrySuffix() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testRegistrySuffix();
}

public void testRegistrySuffixExclusion() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testRegistrySuffixExclusion();
}

public void testRegistrySuffixMultipleUnders() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testRegistrySuffixMultipleUnders();
}

public void testToString() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testToString();
}

public void testTopDomainUnderRegistrySuffix() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testTopDomainUnderRegistrySuffix();
}

public void testTopPrivateDomain() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testTopPrivateDomain();
}

public void testUnderPrivateDomain() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testUnderPrivateDomain();
}

public void testUnderPublicSuffix() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testUnderPublicSuffix();
}

public void testUnderRegistrySuffix() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testUnderRegistrySuffix();
}

public void testUnderTopDomainUnderRegistrySuffix() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testUnderTopDomainUnderRegistrySuffix();
}

public void testValid() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testValid();
}

public void testValidTopPrivateDomain() throws Exception {
  com.google.common.net.InternetDomainNameTest testCase = new com.google.common.net.InternetDomainNameTest();
  testCase.testValidTopPrivateDomain();
}
}
