/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class ToStringHelperTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testConstructorLenient_anonymousClass() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testConstructorLenient_anonymousClass();
}

public void testConstructorLenient_classObject() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testConstructorLenient_classObject();
}

public void testConstructorLenient_innerClass() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testConstructorLenient_innerClass();
}

public void testConstructorLenient_instance() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testConstructorLenient_instance();
}

public void testConstructor_stringObject() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testConstructor_stringObject();
}

public void testToStringHelperLenient_localInnerClass() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringHelperLenient_localInnerClass();
}

public void testToStringHelperLenient_localInnerNestedClass() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringHelperLenient_localInnerNestedClass();
}

public void testToStringHelperLenient_moreThanNineAnonymousClasses() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringHelperLenient_moreThanNineAnonymousClasses();
}

public void testToStringLenient_addValue() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_addValue();
}

public void testToStringLenient_addValueWithNullValue() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_addValueWithNullValue();
}

public void testToStringLenient_addWithNullValue() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_addWithNullValue();
}

public void testToStringLenient_complexFields() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_complexFields();
}

public void testToStringLenient_nullInteger() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_nullInteger();
}

public void testToStringLenient_oneField() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_oneField();
}

public void testToStringLenient_oneIntegerField() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToStringLenient_oneIntegerField();
}

public void testToString_addWithNullName() throws Exception {
  com.google.common.base.ToStringHelperTest testCase = new com.google.common.base.ToStringHelperTest();
  testCase.testToString_addWithNullName();
}
}
