/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class HashBiMapTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testBashIt() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testBashIt();
}

public void testBiMapEntrySetIteratorRemove() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testBiMapEntrySetIteratorRemove();
}

public void testInsertionOrder() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInsertionOrder();
}

public void testInsertionOrderAfterForcePut() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInsertionOrderAfterForcePut();
}

public void testInsertionOrderAfterInverseForcePut() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInsertionOrderAfterInverseForcePut();
}

public void testInsertionOrderAfterRemoveFirst() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInsertionOrderAfterRemoveFirst();
}

public void testInsertionOrderAfterRemoveLast() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInsertionOrderAfterRemoveLast();
}

public void testInsertionOrderAfterRemoveMiddle() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInsertionOrderAfterRemoveMiddle();
}

public void testInverseEntrySetValueNewKey() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInverseEntrySetValueNewKey();
}

public void testInverseInsertionOrderAfterInverse() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInverseInsertionOrderAfterInverse();
}

public void testInverseInsertionOrderAfterInverseForcePut() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInverseInsertionOrderAfterInverseForcePut();
}

public void testInverseInsertionOrderAfterInverseForcePutPresentKey() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testInverseInsertionOrderAfterInverseForcePutPresentKey();
}

public void testMapConstructor() throws Exception {
  com.google.common.collect.HashBiMapTest testCase = new com.google.common.collect.HashBiMapTest();
  testCase.testMapConstructor();
}
}
