/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.primitives;
public class UnsignedLongsTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.primitives.testModule";
}
public void testCompare() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testCompare();
}

public void testDecodeLong() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testDecodeLong();
}

public void testDecodeLongFails() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testDecodeLongFails();
}

public void testDivide() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testDivide();
}

public void testJoin() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testJoin();
}

public void testLexicographicalComparator() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testLexicographicalComparator();
}

public void testMax() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testMax();
}

public void testMax_noArgs() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testMax_noArgs();
}

public void testMin() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testMin();
}

public void testMin_noArgs() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testMin_noArgs();
}

public void testParseLong() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testParseLong();
}

public void testParseLongEmptyString() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testParseLongEmptyString();
}

public void testParseLongFails() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testParseLongFails();
}

public void testParseLongThrowsExceptionForInvalidRadix() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testParseLongThrowsExceptionForInvalidRadix();
}

public void testParseLongWithRadix() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testParseLongWithRadix();
}

public void testParseLongWithRadixLimits() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testParseLongWithRadixLimits();
}

public void testRemainder() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testRemainder();
}

public void testSort() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testSort();
}

public void testSortDescending() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testSortDescending();
}

public void testSortDescendingIndexed() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testSortDescendingIndexed();
}

public void testSortIndexed() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testSortIndexed();
}

public void testToString() throws Exception {
  com.google.common.primitives.UnsignedLongsTest testCase = new com.google.common.primitives.UnsignedLongsTest();
  testCase.testToString();
}
}
