/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class SuppliersTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testCompose() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testCompose();
}

public void testComposeWithLists() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testComposeWithLists();
}

public void testCompose_equals() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testCompose_equals();
}

public void testMemoize() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testMemoize();
}

public void testMemoizeExceptionThrown() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testMemoizeExceptionThrown();
}

public void testMemoize_redudantly() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testMemoize_redudantly();
}

public void testOfInstanceSuppliesNull() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testOfInstanceSuppliesNull();
}

public void testOfInstanceSuppliesSameInstance() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testOfInstanceSuppliesSameInstance();
}

public void testOfInstance_equals() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testOfInstance_equals();
}

public void testSupplierFunction() throws Exception {
  com.google.common.base.SuppliersTest testCase = new com.google.common.base.SuppliersTest();
  testCase.testSupplierFunction();
}
}
