/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class Collections2Test_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testOrderedPermutationSetContains() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetContains();
}

public void testOrderedPermutationSetEmpty() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetEmpty();
}

public void testOrderedPermutationSetOneElement() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetOneElement();
}

public void testOrderedPermutationSetRepeatedElements() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetRepeatedElements();
}

public void testOrderedPermutationSetRepeatedElementsSize() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetRepeatedElementsSize();
}

public void testOrderedPermutationSetSizeOverflow() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetSizeOverflow();
}

public void testOrderedPermutationSetThreeElements() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testOrderedPermutationSetThreeElements();
}

public void testPermutationSetContains() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetContains();
}

public void testPermutationSetEmpty() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetEmpty();
}

public void testPermutationSetFourElements() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetFourElements();
}

public void testPermutationSetOneElement() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetOneElement();
}

public void testPermutationSetSize() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetSize();
}

public void testPermutationSetSizeOverflow() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetSizeOverflow();
}

public void testPermutationSetThreeElements() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetThreeElements();
}

public void testPermutationSetThreeElementsOutOfOrder() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetThreeElementsOutOfOrder();
}

public void testPermutationSetThreeRepeatedElements() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetThreeRepeatedElements();
}

public void testPermutationSetTwoElements() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testPermutationSetTwoElements();
}

public void testToStringImplWithNullEntries() throws Exception {
  com.google.common.collect.Collections2Test testCase = new com.google.common.collect.Collections2Test();
  testCase.testToStringImplWithNullEntries();
}
}
