/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.primitives;
public class ImmutableIntArrayTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.primitives.testModule";
}
public void testBuilder_bruteForce() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testBuilder_bruteForce();
}

public void testBuilder_presize_negative() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testBuilder_presize_negative();
}

public void testBuilder_presize_zero() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testBuilder_presize_zero();
}

public void testContains() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testContains();
}

public void testCopyOf_array_empty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_array_empty();
}

public void testCopyOf_array_nonempty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_array_nonempty();
}

public void testCopyOf_collection_empty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_collection_empty();
}

public void testCopyOf_collection_nonempty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_collection_nonempty();
}

public void testCopyOf_iterable_collection_empty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_iterable_collection_empty();
}

public void testCopyOf_iterable_collection_nonempty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_iterable_collection_nonempty();
}

public void testCopyOf_iterable_notCollection_empty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_iterable_notCollection_empty();
}

public void testCopyOf_iterable_notCollection_nonempty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_iterable_notCollection_nonempty();
}

public void testCopyOf_stream() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testCopyOf_stream();
}

public void testEquals() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testEquals();
}

public void testForEach() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testForEach();
}

public void testGet_bad() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testGet_bad();
}

public void testGet_good() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testGet_good();
}

public void testIndexOf() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testIndexOf();
}

public void testIsEmpty() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testIsEmpty();
}

public void testLastIndexOf() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testLastIndexOf();
}

public void testLength() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testLength();
}

public void testOf0() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf0();
}

public void testOf1() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf1();
}

public void testOf2() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf2();
}

public void testOf3() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf3();
}

public void testOf4() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf4();
}

public void testOf5() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf5();
}

public void testOf6() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf6();
}

public void testOf7() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testOf7();
}

public void testStream() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testStream();
}

public void testSubArray() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testSubArray();
}

public void testTrimmed() throws Exception {
  com.google.common.primitives.ImmutableIntArrayTest testCase = new com.google.common.primitives.ImmutableIntArrayTest();
  testCase.testTrimmed();
}
}
